; 
;
;   file: code05.asm
;   purpose
;       base 64 encoding and decoding procedures
;   extra info, read MASM32 help parts
;       introduction to assembler
;           - registers
;           - working with strings
;       intel opcodes and mnemonics
;
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;   application model
;   ~~~~~~~~~~~~~~~~~

    .386                      ; create 32 bit code
    .model flat, stdcall      ; 32 bit memory model
    option casemap :none      ; case sensitive
 
;   includes
;   ~~~~~~~~

    include \masm32\include\windows.inc
    include \masm32\include\user32.inc
    include \masm32\include\kernel32.inc
    include \masm32\macros\macros.asm


;   libraries
;   ~~~~~~~~~

    includelib \masm32\lib\user32.lib
    includelib \masm32\lib\kernel32.lib

;   prototypes
;   ~~~~~~~~~~

    venc_b64    PROTO :DWORD,:DWORD,:DWORD
    
;   uninitialized data
;   ~~~~~~~~~~~~~~~~~~
;.data?
;    szBuffer db 128 dup (?)

;   initialized data
;   ~~~~~~~~~~~~~~~~

.data
    B64      db "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
    B64PAD   db "="
    ;szString db "ABCDE",0
    ;szString db 255,255,255,255,255
    

.code

; ##########################################################################

LibMain proc hInstDLL:DWORD, reason:DWORD, unused:DWORD
    mov eax, 1
    ret
LibMain Endp

; 
;
;   proc: venc_b64
;
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
venc_b64 proc lpIn:DWORD,dwSizeIn:DWORD,lpOut:DWORD

    ; push registers on stack, make them usable
    push ebx
    push ecx
    push esi
    push edi

    mov esi, lpIn
    mov edi, lpOut
    mov ecx, dwSizeIn
  
venc_loop:

    cmp ecx, 3
    jb  venc_last2

    mov eax, [esi]  ; copy 4 bytes to eax

    ; first b64 char
    movzx ebx, al
    shr bl, 2
    mov dl, byte ptr B64[ebx]
    
    ; second b64 char
    mov bl, ah
    shr bl, 4
    shl al, 6
    shr al, 2
    or  bl, al
    mov dh, byte ptr B64[ebx]

    ; prepare for the last 2
    shl edx, 16
    shr eax, 8

    ; third b64 char    
    shl al, 4
    shr al, 2
    mov bl, ah
    shr bl, 6
    or  bl, al
    mov dl, byte ptr B64[ebx]

    ; last b64 char
    and eax, 00000000000000000011111100000000b
    shr eax, 8
    mov dh, byte ptr B64[eax]        
    
    rol edx, 16

    ; store the encoded bytes in EDI
    mov [edi], edx
    
    ; increment pointers
    add edi, 4   
    add esi, 3

    ; decrement counter
    sub ecx, 3

    ; again
    jmp venc_loop

venc_last2:

    cmp ecx, 2
    jb  venc_last1

    mov eax, [esi]  ; copy 4 bytes to eax

    ; first b64 char
    movzx ebx, al
    shr bl, 2
    mov dl, byte ptr B64[ebx]
    
    ; second b64 char
    mov bl, ah
    shr bl, 4
    shl al, 6
    shr al, 2
    or  bl, al
    mov dh, byte ptr B64[ebx]

    ; prepare for the last 2
    shl edx, 16
    shr eax, 8

    ; third b64 char    
    and eax, 00000000000000000000000000001111b
    shl al, 2
    mov dl, byte ptr B64[eax]

    ; last b64 char
    mov dh, byte ptr B64PAD        
    
    rol edx, 16

    ; store the encoded bytes in EDI
    mov [edi], edx

    add edi, 4   

    jmp venc_end

venc_last1:

    cmp ecx, 1
    jb  venc_end

    mov eax, [esi]  ; copy 4 bytes to eax
   
    ; first b64 char
    movzx ebx, al
    shr bl, 2
    mov dl, byte ptr B64[ebx]
    
    ; second b64 char
    and eax, 00000000000000000000000000000011b
    shl al, 4
    mov dh, byte ptr B64[eax]
    shl edx, 16
    mov dl, byte ptr B64PAD        
    mov dh, byte ptr B64PAD
    rol edx, 16

    mov [edi], edx

    add edi, 4   
    
venc_end:

    xor eax, eax
    pop edi
    pop esi
    pop ecx
    pop ebx

    ret
    
venc_b64 endp

; 

End LibMain