; 
;
;   file: code05.asm
;   purpose
;       learning assembly
;       base 64 encoding and decoding procedures
;   extra info, read MASM32 help parts
;       introduction to assembler
;           - registers
;           - working with strings
;       intel opcodes and mnemonics
;
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;   application model
;   ~~~~~~~~~~~~~~~~~

.386
.model flat, stdcall
option casemap :none

;   includes
;   ~~~~~~~~

include \masm32\include\windows.inc
include \masm32\include\user32.inc
include \masm32\include\kernel32.inc
include \masm32\macros\macros.asm


;   libraries
;   ~~~~~~~~~

includelib \masm32\lib\user32.lib
includelib \masm32\lib\kernel32.lib

;   prototypes
;   ~~~~~~~~~~

venc_b64        PROTO :DWORD,:DWORD,:DWORD
    
;   uninitialized data
;   ~~~~~~~~~~~~~~~~~~
.data?

;buffer  db 8 dup(?)

;   initialized data
;   ~~~~~~~~~~~~~~~~

.data

B64     db "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
B64PAD  db "="
;szTemp  db "ABCDE",0
;szTemp  db 255,255,255,255,255 ;"ABCDE"

 
;   ~code~
;   ~~~~~~

.code
;start:

;    szText szTemp,"ABCDE"
;    invoke venc_b64, ADDR szTemp, 5, ADDR buffer
;    invoke ExitProcess, 0
; ##########################################################################

LibMain proc hInstDLL:DWORD, reason:DWORD, unused:DWORD
    mov eax, 1
    ret
LibMain Endp

; 
;
;   proc: venc_b64
;
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
venc_b64 proc lpIn:DWORD,dwSize:DWORD,lpOut:DWORD

    push ebx
    push ecx
    push esi
    push edi

    cld
    mov esi, lpIn
    mov edi, lpOut
    mov ecx, dwSize
    xor eax, eax

venc_loop:
    cmp ecx, 2
    jbe venc_last2
    lodsb                       ; load byte 1
    mov edx, eax                ; store byte 1 in EDX
    shr eax, 2                  ; shift EAX 2 bits to the right
    mov al , byte ptr B64[eax]  ; B64 char 1
    stosb                       ; store B64 char 1
    lodsb                       ; load byte 2
    mov ebx, eax                ; store byte 2 in EBX
    and eax, 11110000b          ; keep high order 4 bits of EAX
    shr eax, 4                  ; shift EAX 4 bits to the right
    and edx, 00000011b          ; keep low order 2 bits of EDX
    shl edx, 4                  ; shift EDX 4 bits to the left
    or  edx, eax                ; join bits of EDX and EAX
    mov al , byte ptr B64[edx]  ; B64 char 2
    stosb                       ; store B64 char 2
    lodsb                       ; load byte 3
    mov edx, eax                ; store byte 3 in EDX
    and ebx, 00001111b          ; keep lower 4 bits of EBX
    shl ebx, 2                  ; shift EBX 2 bits to the left
    ; shr removes the bits on the right, zero fill
    ;and eax, 11000000b          ; keep high order 2 bits of EAX
    shr eax, 6                  ; shift EAX 6 bits to the right
    or  ebx, eax                ; join bits of EBX and EAX
    mov al , byte ptr B64[ebx]  ; B64 char 3
    stosb                       ; store B64 char 3
    and edx, 00111111b          ; keep low order 6 bits of EDX
    mov al , byte ptr B64[edx]  ; B64 char 4
    stosb                       ; store B64 char 4

    sub ecx, 3                  ; increment counter
    jmp venc_loop               ; again....
venc_last2:
    jb venc_last1

    lodsb                       ; load byte 1
    mov edx, eax                ; store EAX in EDX
    shr eax, 2                  ; shift EAX 2 bits to the right
    mov al , byte ptr B64[eax]  ; B64 char 1
    stosb                       ; store B64 char 1
    lodsb                       ; load byte 2
    mov ebx, eax                ; store EAX in EBX
    shl edx, 8                  ; shift EDX bits 8 to the left
    or  edx, eax                ; join bits of EDX and EAX
    and edx, 1111110000b        ; keep 6 bits of EDX
    and ebx,   00001111b        ; and keep 4 of EBX
    shr edx, 4                  ; shift EDX 4 to the right
    mov al , byte ptr B64[edx]  ; B64 char 2
    stosb                       ; store B64 char 2
    shl ebx, 2                  ; shift EBX 2 bits to the left
    mov al , byte ptr B64[ebx]  ; B64 char 3
    stosb                       ; store B64 char 3
    mov al , byte ptr B64PAD    ; B64 char 4 (padding char)
    stosb                       ; store B64 char 4

    jmp venc_end
venc_last1:
    cmp ecx, 0
    je venc_end

    lodsb                       ; load byte 1
    mov edx, eax                ; store EAX in EDX
    shr eax, 2                  ; shift EAX 2 bits to the right
    mov al , byte ptr B64[eax]  ; B64 char 1
    stosb                       ; store B64 char 1
    and edx, 00000011b          ; keep lower 2 bits of EDX
    shl edx, 4                  ; shift EDX 4 bits to the left
    mov al , byte ptr B64[edx]  ; B64 char 2
    stosb                       ; store B64 char 2
    mov al , byte ptr B64PAD    ; B64 char 3 (padding char)
    stosb                       ; store B64 char 3
    mov al , byte ptr B64PAD    ; B64 char 4 (padding char)
    stosb                       ; store B64 char 4

venc_end:

    xor eax, eax
    pop edi
    pop esi
    pop ecx
    pop ebx

    ret

venc_b64 endp 

; 

;end start

End LibMain